﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

// ReSharper disable once CheckNamespace
namespace HeuristicLab.Algorithms.EGO {

  [StorableClass]
  [Item("UniformRandomDiscreteSampling", "A uniform random sampling strategy for real valued optimization")]
  public class UniformRandomDiscreteSampling : ParameterizedNamedItem, IInitialSampling<IntegerVector> {

    #region HL-Constructors, Serialization and Cloning 
    [StorableConstructor]
    protected UniformRandomDiscreteSampling(bool deserializing) : base(deserializing) { }
    protected UniformRandomDiscreteSampling(UniformRandomDiscreteSampling original, Cloner cloner) : base(original, cloner) { }
    public UniformRandomDiscreteSampling() {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new UniformRandomDiscreteSampling(this, cloner);
    }

    public IntegerVector[] GetSamples(int noSamples, IntegerVector[] existingSamples, IEncoding encoding, IRandom random) {
      var enc = encoding as IntegerVectorEncoding;
      var res = new IntegerVector[noSamples];
      for (var i = 0; i < noSamples; i++) {
        var r = new IntegerVector(enc.Length);
        res[i] = r;
        for (var j = 0; j < enc.Length; j++) {
          var b = j % enc.Bounds.Rows;
          r[j] = UniformRandom(enc.Bounds[b, 0], enc.Bounds[b, 1], random);
        }
      }
      return res;
    }
    private static int UniformRandom(int min, int max, IRandom rand) {
      return rand.Next(min, max + 1);  //TODO check wether max is inclusive or exclusive
    }
  }




  #endregion


}

